#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
""" NAME
        ltc2990.py

    SYNOPSIS
        python module

    DESCRIPTION
        module to manage the ltc2990. Configure it and read sensors.

    OPTIONS
        > ...

    EXAMPLE
        > ...

    AUTHOR
        M Jevaud -- 2016

    lICENSE
    (c) Copyright CERN for the benefit of the LHCb Collaboration.
    Distributed under the terms of the GNU General Public Licence V3.
    ana.py script is free and open source software.

    CHANGELOG
        PY Duval 11/03/2016: code factorisation
        PY Duval 07/11/2017: add methods/names to be used by V2/FC0 in 
                             LLI_PCIe40_devices to comply with pep8 
                             coding rules.        
"""
from lli import i2c
from string import*

import time
import os

# registers' index

# STATUS Register mode Read
STATUS_REGISTER_ADDRESS = 0

# CONTROL Register mode Read/Write
CONTROL_REGISTER_ADDRESS = 1

# TRIGGER Register mode Read Write
# si on ecrit nimporte quelle valeur dans ce registre, on declenche
# une conversion la lecture de ce registre est la valeur du Staus Register
TRIGGER_REGISTER_ADDRESS = 2

# TINTMSB Register mode Read
# Internal Temperature MSB
TINTMSB_REGISTER_ADDRESS = 4

# TINTLSB Register mode Read
# Internal Temperature LCB
TINTLSB_REGISTER_ADDRESS = 5

# V1MSB Register mode Read
# V1, V1-V2, or TR1 MSB
V1MSB_REGISTER_ADDRESS = 6
V1minV2MSB_REGISTER_ADDRESS = 6

# V1LSB Register mode Read
# V1, V1-V2, or TR1 LSB
V1LSB_REGISTER_ADDRESS = 7
V1minV2LSB_REGISTER_ADDRESS = 7

# V2MSB Register mode Read
# V2, V1-V2, or TR1 MSB
V2MSB_REGISTER_ADDRESS = 8

# V2LSB Register mode Read
# V2, V1-V2, or TR1 LSB
V2LSB_REGISTER_ADDRESS = 9

# V3MSB Register mode Read
# V3, V3-V4, or TR2 MSB
V3MSB_REGISTER_ADDRESS = 10
V3minV4MSB_REGISTER_ADDRESS = 10

# V3LSB Register mode Read
# V3, V3-V4, or TR2 LSB
V3LSB_REGISTER_ADDRESS = 11
V3minV4LSB_REGISTER_ADDRESS = 11

# V4MSB Register mode Read
# V4, V3-V4, or TR2 MSB
V4MSB_REGISTER_ADDRESS = 12

# V4LSB Register mode Read
# V4, V3-V4, or TR2 LSB
V4LSB_REGISTER_ADDRESS = 13

# VCCMSB Register mode Read
# VCC MSB
VCCMSB_REGISTER_ADDRESS = 14

# VCCLSB Register mode Read
# VCC LSB
VCCLSB_REGISTER_ADDRESS = 15

# Temporisation Fernand Raynaud pour basculer entre 2 modes
TEMPO_FERNAND = 0.3

# usefull for configuration functions
trigger = 0x7F

def setConfig(dev, bus, add, config):
    """
    1) Apply the configuration
    2) Déclenchement du trigger : on met nimporte quelle valeur dans
       TRIGGER_REGISTER, par exemple 0x7F
       je dis 0x7F car apparement le bit 7 du TRIGGER_REGISTER  reste
       toujours à 0, donc la relecture est erronee...

    3) 13/01/2015 : on met une tempo car lorsque l'on change de mode,
       car on a observé que l'on doit attendre un certain temps
       (au sens etymologique de fernand raynaud...) pour basculer

    dev : the board id
    bus : the i2c bus number on the board
    add : the component i2c address
    config : value to write in the control register
    """
    answer1 = i2c.write(dev, bus, add, CONTROL_REGISTER_ADDRESS, config)[1]
    answer2 = i2c.write(dev, bus, add, TRIGGER_REGISTER_ADDRESS, 0x7F)[1]
    time.sleep(TEMPO_FERNAND)

    if (answer1 == config) and (answer2 == trigger):
        # print ("Configuration is OK") mis en commentaire par MJ le 17/01/2017
        return True
    else:
        # print ("Configuration is not OK!!!")  mis en commentaire par MJ le 17/01/2017
        return False

def set_config(dev, bus, add, config):
    return setConfig(dev, bus, add, config)

def Config2TR(dev, bus, add):
    """
    Cirtcuit configuration for 2 differential temperature probes, mode repeat,
    temperature en Celsius
    CONTROL_REGISTER= 0x1D (0001 1101), c'est à dire:
       Mode[2:0] = 101 = TR1,TR2
       Mode[4:3] = 11  = All Measurements per Mode[2:0], lectures de registres
                         fixees sur TR1,TR2
    Temperature Format = 0 (Temperature reported in Celsius)
    Repeat/Single = 0      (Repeat Acquisition)

    dev : the board id
    bus : the i2c bus number on the board
    add : the component i2c address
    """
    #print ("ltc2990 configuration  for 2 TR sensor temperature is running....") mis en commentaire par MJ le 17/01/2017
    return setConfig(dev, bus, add, 0x1D)

def config2_differential_temps(dev, bus, add):
    return setConfig(dev, bus, add, 0x1D)

def Config2VoltDif(dev, bus, add):
    """
    Circuit configuration for 2 differential tensions reading, mode repeat,
    temperature en Celsius
    CONTROL_REGISTER= 0x1E (0001 1110), c'est à dire:
        Mode[2:0] = 101 = lecture de V1-V2, V3-V4
        Mode[4:3] = 11  = All Measurements per Mode[2:0],lectures de registres
                          fixees sur V1-V2, V3-V4
    Temperature Format = 0 = Temperature reported in Celsius
    Repeat/Single = 0 = Repeat Acquisition

    dev : the board id
    bus : the i2c bus number on the board
    add : the component i2c address
    """
    # print ("ltc2990 configuration  for 2 differential voltage is running....") mis en commentaire par MJ le 17/01/2017
    return setConfig(dev, bus, add, 0x1E)

def config2_differential_tensions(dev, bus, add):
    return setConfig(dev, bus, add, 0x1E)

def Config4VoltSingleEnded(dev, bus, add):
    """
    Circuit configuration to read 4 tensions single ended,  mode repeat,
    température en Celsius
    CONTROL_REGISTER= 0x1F = 0001 1111, c'est à dire:
        Mode[2:0] = 111 = lecture de V1,V2,V3,V4
        Mode[4:3] = 11  = All Measurements per Mode[2:0],lectures de registres
                          fixées sur V1,V2, V3,V4
    Temperature Format = 0 = Temperature reported in Celsius
    Repeat/Single = 0 = Repeat Acquisition

    Note : it is important to select Celsius temp because even in this config
           we read the circuit internal temperature

    dev : the board id
    bus : the i2c bus number on the board
    add : the component i2c address
    """
    # print ("Configuration ltc2990 for 4 single ended voltage is running..") mis en commentaire par MJ le 17/01/2017
    return setConfig(dev, bus, add, 0x1F)

def config4_tensions_single_ended(dev, bus, add):
    return Config4VoltSingleEnded(dev, bus, add)

def two_comp13(nb13bits):
    """
    All temperatures -internal or TR1 or TR2) are in 13bits complemented to 2.
    So we need this conversion function
    """
    if nb13bits > 8191:
        print("Number is greater than 8191!!!")
        nb13bits = 8191
    if nb13bits <= 4095:
        return nb13bits
    else:
        return nb13bits-8192


def two_comp15(nb15bits):
    """
    All single-ended (V1,V2,V3,V4) or differential (V1-V2, V3-V4) tensions
    are in 15bits complemented to 2.
    So we need this conversion function
    """
    if nb15bits > 32767:
        print("Number is greater than 32767!!!")
        nb15bits = 32767
    if nb15bits <= 16383:
        return nb15bits
    else:
        return nb15bits-32768


def enum(**named_values):
    return type('Enum', (), named_values)


Measure = enum(TEMP='temp', VOLT='tension')


def ReadTempOrTension(dev, bus, add, msbReg, lsbReg, measureType, inversion):
    """
    We loop reading the MSB register untill bit 7 switch to 1.
    We apply the mask:
       - when temperature reading to extract D8-D12 bits
       - when tension reading D8-D13 and D14 (sign)
    We read the LSB register to concatenate the two words to get the full value
    We then complement the value to 2 and multiply by a constant factor
       - factor = 0.0625 for temp
       - factor = 0.00030518 for tensions

    dev         : the board id
    bus         : the i2c bus number on the board
    add         : the component i2c address
    msbReg      : the msb register index
    lsbReg      : the lsb register index
    measureType : type of measure Measure.TEMP or Measure.VOLT
    inversion   : for some components the tension is inverted (False or True)
    """
    cpt_timeout = 0
    DataValidBit = 0
    if (measureType == Measure.TEMP):
        mask = 0b11111
        factor = 0.0625
    else:
        mask = 0b1111111
        factor = 0.00030518

# On divise ensuite par la valeur de la resistance de shunt (
# supposee egale a 0.003 ohm) pour avoir la valeur du courant IPEX09V
    if inversion:
       RSHUNT = 0.003
       LSB_DIFFERENTIAL = 0.00001942
       factor = LSB_DIFFERENTIAL/RSHUNT		

    while not(DataValidBit):
        LocalTempMSB = i2c.read(dev, bus, add, msbReg)[1]
        DataValidBit = (0b10000000)&LocalTempMSB
        cpt_timeout=cpt_timeout+1
        if cpt_timeout > 1000:
            print ("Error Timeout to read")
            break; 
    dataMSB = LocalTempMSB & mask	
    dataLSB = i2c.read(dev, bus, add, lsbReg)[1] 
    dataInteger16 = dataMSB*256 + dataLSB
    if ( measureType == Measure.TEMP):
        return two_comp13(dataInteger16)*factor
    else:
        if (inversion):
            return -two_comp15(dataInteger16)*factor
        else:
            return two_comp15(dataInteger16)*factor

def read_temperatures_or_tensions(dev, bus, add, msbReg, lsbReg, measureType, inversion):
    return ReadTempOrTension(dev, bus, add, msbReg, lsbReg, measureType, inversion)

def ReadLocalTemperature(dev, bus, add):
    return ReadTempOrTension(dev, bus, add, TINTMSB_REGISTER_ADDRESS, TINTLSB_REGISTER_ADDRESS, Measure.TEMP, False)
def read_local_temperature(dev, bus, add):
    return ReadLocalTemperature(dev, bus, add)

def ReadTR1(dev, bus,add):
    return ReadTempOrTension(dev, bus, add, V1MSB_REGISTER_ADDRESS, V1LSB_REGISTER_ADDRESS, Measure.TEMP, False)
def read_v1_temperature(dev, bus,add):
    return ReadTR1(dev, bus,add)

def ReadTR2(dev, bus,add):
    return ReadTempOrTension(dev, bus, add, V3MSB_REGISTER_ADDRESS, V3LSB_REGISTER_ADDRESS, Measure.TEMP, False)
def read_v3_temperature(dev, bus,add):
    return ReadTR2(dev, bus,add)

def ReadV1(dev, bus,add):
    return ReadTempOrTension(dev, bus, add, V1MSB_REGISTER_ADDRESS, V1LSB_REGISTER_ADDRESS, Measure.VOLT, False)
def read_v1_tension(dev, bus,add):
    return ReadV1(dev, bus,add)

def ReadV2(dev, bus,add):
   return ReadTempOrTension(dev, bus, add, V2MSB_REGISTER_ADDRESS, V2LSB_REGISTER_ADDRESS, Measure.VOLT, False)
def read_v2_tension(dev, bus,add):
    return ReadV2(dev, bus,add)

def ReadV3(dev, bus,add):
    return ReadTempOrTension(dev, bus, add, V3MSB_REGISTER_ADDRESS, V3LSB_REGISTER_ADDRESS, Measure.VOLT, False)
def read_v3_tension(dev, bus,add):
    return ReadV3(dev, bus,add)

def ReadV4(dev, bus,add):
    return ReadTempOrTension(dev, bus, add, V4MSB_REGISTER_ADDRESS, V4LSB_REGISTER_ADDRESS, Measure.VOLT, False)
def read_v4_tension(dev, bus,add):
    return ReadV4(dev, bus,add)

def ReadIPEX09V(dev, bus,add):
   """
   Entering this function we have to be in read differential tension mode.
   """
   return ReadTempOrTension(dev, bus, add, V1minV2MSB_REGISTER_ADDRESS, V1minV2LSB_REGISTER_ADDRESS, Measure.VOLT, True)


def ReadIPEX09VDDA(dev, bus,add):
   """
   Entering this function we have to be in read differential tension mode.
   """
   return ReadTempOrTension(dev, bus, add, V3minV4MSB_REGISTER_ADDRESS, V3minV4LSB_REGISTER_ADDRESS, Measure.VOLT, True)

dev = 0
bus = 1

def main():
   # config for comp1
   print "============= COMP 1 =================="
   add = 0x4E
   Config2TR(dev, bus, add)
   print "TR1 " + str(ReadTR1(dev, bus, add))
   print "TR2 " + str(ReadTR2(dev, bus, add))

   Config4VoltSingleEnded(dev, bus, add)
   print "V1 " + str(ReadV1(dev, bus, add))
   print "V2 " + str(ReadV2(dev, bus, add))
   print "V3 " + str(ReadV3(dev, bus, add))
   print "V4 " + str(ReadV4(dev, bus, add))

   Config2VoltDif(dev, bus, add)
   print "IPEX09V " + str(ReadIPEX09V(dev, bus,add))
   print "IPEX09VDDA " + str(ReadIPEX09VDDA(dev, bus,add))
   # end config for comp 1
   # config for comp2
   print "============= COMP 2 =================="
   add = 0x4D
   Config4VoltSingleEnded(dev, bus, add)
   print "Local temp " + str(ReadLocalTemperature(dev, bus, add))

   Config4VoltSingleEnded(dev, bus, add)
   print "V1 " + str(ReadV1(dev, bus, add))
   print "V2 " + str(ReadV2(dev, bus, add))
   print "V3 " + str(ReadV3(dev, bus, add))
   print "V4 " + str(ReadV4(dev, bus, add))

   Config2VoltDif(dev, bus, add)
   print "IPEX09V " + str(ReadIPEX09V(dev, bus,add))
   print "IPEX09VDDA " + str(ReadIPEX09VDDA(dev, bus,add))
   # end config for comp2
   # config for comp3
   print "============= COMP 3 =================="
   add = 0x4F
   Config2TR(dev, bus, add)
   print "TR1 " + str(ReadTR1(dev, bus, add))
   print "TR2 " + str(ReadTR2(dev, bus, add))

   Config4VoltSingleEnded(dev, bus, add)
   print "V1 " + str(ReadV1(dev, bus, add))
   print "V2 " + str(ReadV2(dev, bus, add))
   print "V3 " + str(ReadV3(dev, bus, add))
   print "V4 " + str(ReadV4(dev, bus, add))

   Config2VoltDif(dev, bus, add)
   print "IPEX09V " + str(ReadIPEX09V(dev, bus,add))
   print "IPEX09VDDA " + str(ReadIPEX09VDDA(dev, bus,add))
   # end config for comp 3

if __name__ == "__main__":
    main()


 




    

        
